      SUBROUTINE LPDUAL (A,KT,RHS,COST,INPUT,TOL,TITLE,PFILE,          
     *   OBJ,X,JX,PI,DJ,ERR,IOUT,WS)  
C
C
C A     --- REAL CONSTRAINT MATRIX OF SIZE AT LEAST M BY N. 
C KT    --- INTEGER VECTOR OF CONSTRAINT TYPES OF SIZE AT LEAST M.
C           -1,0,+1  MEAN  .GE.,.EQ.,.LE.  RESPECTIVELY
C RHS   --- REAL VECTOR OF RIGHT-HAND-SIDE VALUES OF SIZE AT LEAST M. 
C COST  --- REAL VECTOR OF COSTS OF SIZE AT LEAST N IF NON-ZERO COSTS
C           ARE NOT ASSIGNED TO SLACKS, AND AT LEAST N+M IF THEY ARE. 
C INPUT --- INTEGER FIXED CONSTANTS VECTOR OF LENGTH AT LEAST 22
C           CONTAINING   
C ML      1 ROW DIMENSION OF A IN CALLING PROGRAM.  MINIMUM IS M.
C NL      2 COLUMN DIMENSION OF A IN CALLING PROGRAM.  MINIMUM IS N.
C M       3 NUMBER OF CONSTRAINTS.
C N       4 NUMBER OF VARIABLES.
C MINMAX  5 =0 IF OBJECTIVE IS TO BE MINIMIZED, 
C           =1 IF OBJECTIVE IS TO BE MAXIMIZED. 
C LENWS   6 DIMENSION OF WS IN CALLING PROGRAM.  MINIMUM IS
C           M**2 + 3*M + 2.
C INFLAG  7 BASIS CONDITION FLAG.
C           =0 NEW PROBLEM, LPDUAL FINDS INITIAL BASIS.
C           =1 NEW PROBLEM, FULL OR PARTIAL INITIAL BASIS SPEC. IN JX.
C           =2 CHANGES IN RHS AND COST ONLY.
C           =3 CHANGES IN RHS ONLY.
C           =4 CHANGES IN COST ONLY.      
C IALLEQ  8 ALL EQUALITY CONSTRAINTS FLAG.
C           =0 ALL CONSTRAINTS ARE EQUALITIES.
C           =1 CONSTRAINT TYPES ARE SPECIFIED IN KT.
C KSLACK  9 ASSIGN ZERO COSTS TO SLACK VARIABLES FLAG.
C           =0 NO.
C           =1 YES.
C NCUT   10 ITERATION LIMIT.  IF .LE. 0, RESET TO 1000.
C NEQ    11 CONSECUTIVE EUQAL OBJECTIVE VALUE LIMIT.  IF .LE. 0,
C           RESET TO 1000.
C NVER   12 REINVERSION FREQUENCY.  IF .LE. 0, RESET TO 30.
C ITRYO  13 REINVERT AT OPTIMUM FLAG.
C           =0 NO.
C           =1 YES.
C IPRNT  14 OUTPUT PROBLEM PARAMETERS OPTION.
C           =0 NO.
C           =1 YES.
C IPDATA 15 OUTPUT PROBLEM DATA OPTION.
C           =0 NO.
C           =1 OUTPUT WITH A MATRIX IN DENSE FORM.
C           =2 OUTPUT WITH A MATRIX IN SPARSE FORM. 
C ILINE  16 OUTPUT INTERMEDIATE OUTPUT. 
C           =0 NO.
C           =1 YES.
C JFINAL 17 OUTPUT SOLUTION REPORT OPTION.
C           =0 NO.
C           =1 OUTPUT SOLUTION REPORT WITHOUT SHADOW PRICES (PI).
C           =2 OUTPUT SOLUTION REPORT WITH SHADOW PRICES (PI).
C JFILE  18 OPTION TO PRINT OR FILE OUTPUT.
C           =0 PRINT ALL OUTPUT.
C           =1 PRINT PROB. PARAM. AND SOLN. REP. AND FILE OTHER OUTPUT.        
C           =2 FILE ALL OUTPUT. 
C JWIDTH 19 MAXIMUM WIDTH OF OUTPUT LINES.  A MINIMUM OF 72 AND A
C           MAXIMUM OF 132 WILL BE USED.
C MX     20 IF POSITIVE, MAXIMUM NUMBER OF CONSTRAINTS THAT MAY BE 
C           ADDED TO BASE PROBLEM.
C IADD   21 IF POSITIVE, NUMBER OF CONSTRAINTS TO ADD TO PROBLEM.          
C IDROP  22 IF POSITIVE, INDEX OF CONSTRAINT TO BE DROPPED FROM PROBLEM.        
C TOL   --- REAL TOLERANCE VECTOR OF LENGTH AT LEAST 5 CONTAINING
C TPIV    1 PIVOT TOLERANCE.   IF .LE. 0, RESET TO 1.E-7. 
C TCOST   2 REDUCED COSTS TOLERANCE.   IF .LE. 0, RESET TO 1.E-6.
C TFEAS   3 FEASIBILITY TOLERANCE.   IF .LE. 0, RESET TO 1.E-6.
C TX      4 ROUND-OFF TOLERANCE.   IF .LE. 0, RESET TO 1.E-10.
C TART    5 LARGE NUMBER FOR ARTIFICIALS.  IF .LE. 0, RESET TO 1.E+8.
C TITLE --- CHARACTER TITLE USED IN OUTPUT.
C           MAY CONTAIN UP TO 64 CHARACTERS.
C           TRUNCATED TO 64 CHARACTERS IF LONGER.
C PFILE --- CHARACTER FILE NAME LPDUAL WRITES OUTPUT ONTO IF REQUESTED.        
C           MAY CONTAIN UP TO 64 CHARACTERS.
C           IF FILE CANNOT BE OPENED, A FILE WILL BE CREATED
C           AND A MESSAGE PRINTED.  IF THE NAME IS ALL BLANKS,
C           OUTPUT TO BE FILED WILL BE DISCARDED. 
C OBJ   --- DOUBLE PRECISION OBJECTIVE VALUE. 
C X     --- DOUBLE PRECISION VECTOR OF LENGTH AT LEAST M+1 THAT WILL
C           CONTAIN THE VALUES OF THE BASIC VARIABLES OF THE SOLUTION.
C           USED IN CONJUNCTION WITH JX TO DETERMINE SOLUTION VECTOR.
C           THE LAST LOCATION ARE NEEDED WHEN AN ARTIFIAL CONSTRAINT 
C           REQUIRED.
C JX    --- INTEGER VECTOR OF LENGTH AT LEAST M+1 INDICATING INDICES OF
C           VARIABLES IN BASIS.  THE LAST LOCATION IS NEEDED WHEN AN
C           ARTIFICIAL CONSTRAINT IS REQUIRED.
C           1 - N ARE STRUCTURALS, N+1 - N+M ARE SLACKS, 0 IS ARTIF.
C           SPECIFIES INITIAL BASIS IF INPUT(7)=1.
C PI    --- DOUBLE PRECISION VECTOR OF LENGTH AT LEAST M+1 THAT WILL
C           CONTAIN THE SHADOW PRICES.  THE LAST LOCATION IS NEEDED WHEN
C           AN ARTIFICIAL CONSTRAINT IS REQUIRED.     
C DJ    --- DOUBLE PRECISION VECTOR THAT WILL CONTAIN THE REDUCED COSTS
C           OF LENGTH AT LEAST M+N IF ANY INEQUALITY CONSTRAINTS ARE
C           PRESENT OR N FOR ALL EQUALITY PROBLEMS.
C ERR   --- REAL ERRORS VECTOR OF LENGTH AT LEAST 4 THAT WILL CONTAIN
C ROWMAX  1 MAXIMUM ROW ERROR.
C ROWSUM  2 SUM OF ROW ERRORS.
C COLMAX  3 MAXIMUM COLUMN ERROR.
C COLSUM  4 SUM OF COLUMN ERRORS.
C IOUT  --- INTEGER VECTOR OF LENGTH AT LEAST 4 THAT WILL CONTAIN
C K       1 STATUS CODE FROM 1 TO 6.
C           1 - SOLUTION IS OPTIMAL.  
C           2 - NO FEASIBLE SOLUTION.
C           3 - UNBOUNDED SOLUTION.
C           4 - ITERATION LIMIT REACHED.  
C           5 - EQUAL CONSECUTIVE OBJECTIVE LIMIT REACHED.
C           6 - PROBLEM DATA ERROR.
C           7 - ALGORITHM ERROR. 
C ITER    2 NUMBER OF ITERATIONS.
C INVC    3 NUMBER OF ITERATIONS SINCE LAST INVERSION.
C NUMVR   4 NUMBER OF INVERSIONS.
C WS    --- DOUBLE PRECISION WORK SPACE ARRAY OF SIZE AT LEAST
C           M**2 + 5*M + 6.
C           THE FIRST (M+2)*(N+2) LOCATIONS OF WS CONTAIN THE
C           INVERSE OF THE BASIS MATRIX.
C
C     VARIABLES OCCURING IN COMMON BLOCKS.
C  
C     DJ1    - DJ OF COLUMN JCOL AFTER CALL TO LPDDJD.
C     EP     - VALUE USED IN LPDMIN.
C     ERRS   - COPY OF ERR.
C     ICON   - CONTINUATION FLAG. 
C     INCOL  - ENTERING COLUMN.
C     INPUTS - COPY OF INPUT.
C     IO1    - UNIT NUMBER FOR STANDARD PRINT.
C     IO2    - UNIT NUMBER USED FOR PROBLEM PARAM. AND SOLN. REPORT.
C     IO3    - UNIT NUMBER USED FOR PROBLEM DATA AND INTERM. OUTPUT.
C     IOUTS  - COPY OF IOUT.
C     IPIV   - PIVOT FLAG.
C     IROW   - ROW PIVOT INDEX.
C     IROW2  - INFEASIBLE ROW, IF ANY.
C     ITRY   - CUTOFF COUNTER.
C     IVER   - RE-INVERSION FLAG. 
C     JCOL   - CURRENT COLUMN.
C     JOUT   - EXITING COLUMN.
C     LENREQ - REQUIRED LENGTH OF WORKSPACE ARRAY WS. 
C     M1     - M IF ARTIF. CONSTRAINT IS NOT USED, ELSE M + 1.
C     MLE    - ROW DIM. OF BASIS INVERSE, SHOULD BE M' + MX + 1
C              WHERE M' IS THE NUMBER OF CONSTRAINTS IN BASE PROBLEM.
C     N1     - N FOR ALL-EQUALITY PROBLEMS, ELSE N + M.
C     NCALL  - NO. CALLS MADE TO LPDUAL IN PROGRAM.  
C     NCOST  - N + M IF SLACK COSTS ASSIGNED, ELSE N. 
C     NEQL   - NO. EQUAL OBJECTIVE VALUES.
C     NUMNZ  - NOT USED.
C     OB     - OBJECTIVE VALUE OF PREVIOUS ITERATION. 
C     OBVAL  - CURRENT OBJECTIVE VALUE.
C     PFILES - COPY OF PFILE.
C     RAT    - PIVOT RATIO.
C     SUMINF - SUM OF INFEASIBILITIES.  
C     TITLES - COPY OF TITLE.
C     TOLS   - COPY OF TOL.
C     XMOST  - MOST NEGATIVE X DURING ITERATION.
C     YJ1    - YJ OF COLUMN JCOL AFTER CALL TO LPDYJD.
C
      INTEGER  KT,INPUT,JX,IOUT
      REAL  A,RHS,COST,TOL,ERR
      DOUBLE PRECISION  OBJ,X,PI,DJ,WS
      CHARACTER*(*)  TITLE,PFILE
C
      DIMENSION  A(*),KT(*),RHS(*),COST(*),INPUT(*),TOL(*),
     1   X(*),JX(*),PI(*),DJ(*),ERR(*),IOUT(*),WS(*)
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C
      CHARACTER*64  TITLES,PFILES
      COMMON /LPDZZC/ TITLES,PFILES
C
      INTEGER  I,L1,L2,L3,LUPRNT,LUFILE,NCALL0
C
C
C     LPDUAL CALLS COUNTER
      SAVE NCALL0
      DATA NCALL0  / 0 /
C
C     OUTPUT UNITS FOR LINE PRINTER (TERMINAL) AND OUTPUT FILE
      DATA  LUPRNT, LUFILE  / 6, 90 /
C
C 
C     INITIALIZATIONS
C
      NCALL0 = NCALL0 + 1
      NCALL = NCALL0
      DO 10 I = 1, 4
         TOLS(I) = TOL(I)
         ERRS(I) = 0.0
         IOUTS(I) = 0
   10 CONTINUE
      TOLS(5) = TOL(5)
      DO 20 I = 1, 22
         INPUTS(I) = INPUT(I)
   20 CONTINUE
      IF (MX .LT. 0) MX = 0
      IF (IADD .LT. 0) IADD = 0
      IF (IDROP .LT. 0) IDROP = 0
      IF (IADD .EQ. 0 .AND. IDROP .EQ. 0) MLE = M + MX + 1
      M1 = M
      NCOST = N + M*KSLACK
      N1 = N + M*IALLEQ
      ICON = 1
      IPIV = 0
      ITRY = 1
      IVER = 0
      NEQL = 0
      TITLES = 
     *'                                                                ' 
      L1 = LEN(TITLE)
      TITLES(1:L1) = TITLE(1:L1)
C
C     SET DEFAULT VALUES.
C
      IF (NCUT .LE. 0) NCUT = 1000
      IF (NEQ .LE. 0) NEQ = 1000
      IF (NVER .LE. 0) NVER = 30
      IF (JWIDTH .LT. 72) JWIDTH = 72
      IF (JWIDTH .GT. 132) JWIDTH = 132
      IF (TPIV .LE. 0.0) TPIV = 1.0E-7
      IF (TCOST .LE. 0.0) TCOST = 1.0E-6
      IF (TFEAS .LE. 0.0) TFEAS = 1.0E-6
      IF (TX .LE. 0.0) TX = 1.0E-10
      IF (TART .LE. 0.0) TART = 1.0E+8
C
C     SET OUTPUT UNITS AND OPEN PRINT FILE IF REQUIRED.
C
      IO1 = LUPRNT
      IF (JFILE .NE. 1 .AND. JFILE .NE. 2) GO TO 120
      PFILES =
     *'                                                                ' 
      L1 = LEN(PFILE)
      L2 = 0
      L3 = 0
      DO 30 I = 1, L1
         IF (PFILE(I:I) .EQ. ' ') GO TO 30
         L2 = L2 + 1
         PFILES(L2:L2) = PFILE(I:I)
   30 CONTINUE
      IF (L2 .GT. 0) GO TO 40
C     BLANK FILE NAME, DISCARD OUTPUT TO BE FILED.
      IPDATA = 0
      ILINE = 0
      IF (JFILE .EQ. 2) GO TO 130
      IPRNT = 0
      JFINAL = 0
      GO TO 50
C     OPEN PRINT FILE
   40 CONTINUE
      OPEN (LUFILE,FILE=PFILES,STATUS='UNKNOWN',ERR=60)
      IO3 = LUFILE 
      IF (JFILE .EQ. 1) GO TO 50
      IO2 = LUFILE
      GO TO 130
   50 CONTINUE
      IO2 = LUPRNT
      GO TO 130
C     CANNOT OPEN FILE. MAKE ONE UP.
   60 CONTINUE
      IF (L3 .EQ. 1) GO TO 100
      L3 = 1
      WRITE (IO1,70) NCALL,PFILES(1:L2) 
   70 FORMAT ( //// 5X,'LPDUAL  CALL',I4
     1           // 1X,'*** UNABLE TO OPEN THE FILE ', A)
      WRITE (PFILES,80) NCALL
C     FORM FILE NAME.
   80 FORMAT ('LPDU',I4.4,'.LIS',52X)
      WRITE (IO1,90) PFILES(1:12) 
   90 FORMAT ( / 5X,'OUTPUT WILL BE FILED ON ',A12) 
      GO TO 40
C     CANNOT OPEN THIS FILE EITHER.
  100 CONTINUE
      WRITE (IO1,110) LUFILE
  110 FORMAT ( / 5X,'CANNOT OPEN THIS FILE EITHER.  CHECK YOUR FILES ',
     1   'FOR THE ABOVE NAME' / 5X,'AND MAKE SURE LOGICAL UNIT',I4,
     2   ' IS CLOSED.' / )
      K = 6
      GO TO 200 
  120 CONTINUE
      IO2 = LUPRNT
      IO3 = LUPRNT
  130 CONTINUE
C
C     PARTITION WORKSPACE ARRAY.
C
      L1 = 1 + MLE**2
      LENREQ = L1 + MLE - 1
C
C     CHECK INPUT DATA PARAMETERS FOR CONSISTENCY.
C
      CALL LPDCHK (KT,JX)
C
      IF (K .EQ. 6) GO TO 200
C
C     QUIT IF BAD VALUES (K = 6)
C
C     OUTPUT PROBLEM PARAMETERS AND PROBLEM DATA IF REQUESTED.
C
      IF (IPRNT .NE. 0) CALL LPDW10
      IF (IPDATA .NE. 0) CALL LPDW20 (A,KT,RHS,COST)
C
C     OUTPUT HEADER FOR INTERMEDIATE OUTPUT, IF ANY EXPECTED.
C
      IF (ILINE .NE. 0) CALL LPDW30
C
C     CALL SECOND LEVEL CONTROL ROUTINE TO SOLVE PROBLEM.
C
      CALL LPDUL2 (A,KT,RHS,COST,X,JX,PI,DJ,WS(1),WS(L1))
C
C     OUTPUT FINAL STATUS FOR INTERMEDIATE OUTPUT, IF ANY.
C
      IF (ILINE .NE. 0) CALL LPDW32 
C
C     OUTPUT SOLUTION REPORT IF REQUESTED.
C
      IF (JFINAL .NE. 0) CALL LPDW40 (X,JX,PI)
C
C     SET EXIT VALUES, IOUT.
C
  200 CONTINUE 
      DO 210 I = 1, 4
         ERR(I) = ERRS(I)
         IOUT(I) = IOUTS(I)
  210 CONTINUE
      OBJ = OBVAL  
C
C     CLOSE PRINT FILE IF REQUIRED.
C
      IF (JFILE .GT. 0) CLOSE (LUFILE,STATUS='KEEP')
C
      RETURN
C
      END


C
C-------------------------------------------------------------------------
C

      SUBROUTINE LPDCHK (KT,JX) 
C
C     CHECK INPUT DATA FOR LPDUAL.
C
      INTEGER KT,JX
      DIMENSION KT(*),JX(*)
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      INTEGER  I,INEQ,ITITLE,J,MT,ZERONE,ZERO12
      LOGICAL  NOGOOD
      DIMENSION  ZERONE(6),ZERO12(3),NOGOOD(23)
C   
      DATA ZERONE  / 5,8,9,13,14,16 /
      DATA ZERO12  / 15,17,18 /
C
C
      ITITLE = 0  
C
C     CHECK KT, GET COUNT ON NO. INEQUALITIES.
C
      MT = 0
      IF (IALLEQ .EQ. 0) GO TO 50
      DO 10 I = 1, M
         INEQ = IABS(KT(I))
         IF (INEQ .GT. 1) GO TO 20  
         MT = MT + INEQ
   10 CONTINUE  
      GO TO 50  
   20 CONTINUE
      IF (ITITLE .EQ. 0) WRITE (IO1,30) NCALL
   30 FORMAT ( //// 5X,'LPDUAL CALL',I4)
      WRITE (IO1,40) (KT(I),I=1,M) 
   40 FORMAT ( / 1X,'*** ERROR, ILLEGAL VALUES IN KT.  KT CONTAINS' 
     1  / (6I12) )
      K = 6
      ITITLE = 1
C  
C     CHECK JX IF NECESSARY.
C
   50 CONTINUE
      IF (INFLAG .EQ. 0) GO TO 90
      DO 60 I = 1, M
         IF (JX(I) .LT. 0 .OR. JX(I) .GT. N1) GO TO 70
   60 CONTINUE
      GO TO 90
   70 CONTINUE  
      IF (ITITLE .EQ. 0) WRITE (IO1,30) NCALL
      WRITE (IO1,80) N1,(JX(I),I=1,M) 
   80 FORMAT ( / 1X,'*** ERROR, INPUT(7) NON-ZERO AND JX CONTAINS ',
     1   'COLUMN NUMBERS' / 5X,'THAT ARE NEGATIVE OR EXCEED',I4,
     2   '.  JX CONTAINS' / (6I12) )
      K = 6
      ITITLE = 1  
C
C     CHECK INPUT.
C
   90 CONTINUE
      NOGOOD(1) = (M .LT. 1 .OR. M .GT. ML)
      NOGOOD(2) = (N .LT. 1 .OR. N .GT. NL)
      NOGOOD(3) = (M .GT. N + MT)
      NOGOOD(4) = (LENWS .LT. LENREQ) 
      NOGOOD(5) = (INFLAG .LT. 0 .OR. INFLAG .GT. 4)
      NOGOOD(6) = (KSLACK .EQ. 1 .AND. IALLEQ .EQ. 0)
      NOGOOD(7) = (NCALL .EQ. 1 .AND. INFLAG .GT. 1)
      NOGOOD(8) = (IADD .GT. 0 .AND. IDROP .GT. 0)
      NOGOOD(9) = (NCALL .GT. 1 .AND. M .GE. MLE)
      NOGOOD(10) = (IDROP .GT. M)
      NOGOOD(11) = (IADD .GT. 0 .AND. INFLAG .GT. 0)
      NOGOOD(12) = (IDROP .GT. 0 .AND. INFLAG .GT. 0)
      NOGOOD(13) = (NCALL .EQ. 1 .AND. IADD .GT. 0)
      NOGOOD(14) = (NCALL .EQ. 1 .AND. IDROP .GT. 0)
      DO 100 I = 1, 6
         J = ZERONE(I)
         NOGOOD(I+14) = (INPUTS(J) .NE. 0 .AND. INPUTS(J) .NE. 1)
  100 CONTINUE
      DO 110 I = 1, 3
         J = ZERO12(I)
         NOGOOD(I+20) = (INPUTS(J) .LT. 0 .OR. INPUTS(J) .GT. 2)
  110 CONTINUE
      DO 120 I = 1, 23  
         IF (NOGOOD(I)) GO TO 130
  120 CONTINUE  
      GO TO 400
C  
  130 CONTINUE
      K = 6
      IF (ITITLE .EQ. 0) WRITE (IO1,30) NCALL
      WRITE (IO1,140) (INPUTS(I),I=1,22)
  140 FORMAT ( / 1X,'*** ERROR, INCONSISTENT DATA IN INPUT.' 
     1  '  INPUT CONTAINS' / 6I12 / 6I12 / 6I12 / 4I12 / 
     2  / 8X,'THE FOLLOWING ERRORS OCCURRED.' / )
      IF (NOGOOD(1)) WRITE (IO1,150)
  150 FORMAT (5X,'INPUT(3) NON-POSITIVE OR EXCEEDS INPUT(1)')
      IF (NOGOOD(2)) WRITE (IO1,160)
  160 FORMAT (5X,'INPUT(4) NON-POSITIVE OR EXCEEDS INPUT(2)')
      IF (NOGOOD(3)) WRITE (IO1,170)
  170 FORMAT (5X,'INPUT(3) EXCEEDS INPUT(4) + (NO. OF NON-ZERO KT(I))')
      IF (NOGOOD(4)) WRITE (IO1,180) LENREQ
  180 FORMAT (5X,'INPUT(6) TOO SMALL.  FOR THIS PROBLEM,',
     1   ' MUST BE AT LEAST',I8)
      IF (NOGOOD(5)) WRITE (IO1,190) 
  190 FORMAT (5X,'INPUT(7) NOT 0, 1, 2, 3, OR 4')
      IF (NOGOOD(6)) WRITE (IO1,200)
  200 FORMAT (5X,'INPUT(8) = 0 AND INPUT(9) = 1')
      IF (NOGOOD(7)) WRITE (IO1,210)
  210 FORMAT (5X,'INPUT(7) EXCEEDS 1 ON FIRST CALL TO LPDUAL')
      IF (NOGOOD(8)) WRITE (IO1,220)
  220 FORMAT (5X,'INPUT(21) POSITIVE AND INPUT(22) POSITIVE')
      IF (NOGOOD(9)) WRITE (IO1,230)
  230 FORMAT (5X,'INPUT(3) EXCEEDS (INPUT(3)+INPUT(20) FOR ORIGINAL',
     1   ' PROBLEM)')
      IF (NOGOOD(10)) WRITE (IO1,240)
  240 FORMAT (5X,'INPUT(22) EXCEEDS INPUT(3)')
      IF (NOGOOD(11)) WRITE (IO1,250)
  250 FORMAT (5X,'INPUT(21) POSITIVE AND INPUT(7) EXCEEDS 1')
      IF (NOGOOD(12)) WRITE (IO1,260)
  260 FORMAT (5X,'INPUT(22) POSITIVE AND INPUT(7) EXCEEDS 1')
      IF (NOGOOD(13)) WRITE (IO1,270)
  270 FORMAT (5X,'INPUT(21) POSITIVE ON FIRST CALL TO LPDUAL')
      IF (NOGOOD(14)) WRITE (IO1,280)
  280 FORMAT (5X,'INPUT(22) POSITIVE ON FIRST CALL TO LPDUAL')
      DO 300 I = 1, 6
         J = ZERONE(I)
         IF (NOGOOD(I+14)) WRITE (IO1,310) J
  300 CONTINUE  
  310 FORMAT (5X,'INPUT(',I2,') NOT 0 OR 1')
      DO 320 I = 1, 3
         J = ZERO12(I)
         IF (NOGOOD(I+20)) WRITE (IO1,330) J
  320 CONTINUE
  330 FORMAT (5X,'INPUT(',I2,') NOT 0, 1, OR 2')
C
  400 CONTINUE
      RETURN
C
      END


C
C-------------------------------------------------------------------------
C


      SUBROUTINE LPDUL2 (A,KT,RHS,COST,X,JX,PI,DJ,E,Y) 
C
C     SECOND LEVEL CONTROL ROUTINE FOR LPDUAL.
C
      INTEGER  KT,JX
      REAL  A,RHS,COST 
      DOUBLE PRECISION  X,PI,DJ,E,Y
      DIMENSION  A(*),KT(*),RHS(*),COST(*),X(*),JX(*),PI(*),DJ(*),
     1   E(*),Y(*)
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C
      INTEGER  J,JUMP
C
C
C     CHANGE SIGN OF COSTS FOR MAX. PROBLEM.
C
      IF (MINMAX .EQ. 0) GO TO 20
      DO 10 J = 1, NCOST
         COST(J) = -COST(J)
   10 CONTINUE
C
C     MODIFY PROBLEM AND INVERSE IF ADDING OR DROPPING CONSTRAINTS.
C    
   20 CONTINUE
      IF (IDROP .EQ. 0 .AND. IADD .EQ. 0) GO TO 30
      CALL LPDMDF (A,KT,RHS,COST,X,JX,PI,E,Y,DJ)
C
C     QUIT IF ALGORITHM ERROR.
C
      IF (K .EQ. 7) GO TO 130
      INFLAG = 2
   30 CONTINUE
      JUMP = INFLAG + 1
      GO TO (80, 90, 40, 50, 40), JUMP
C
C     MODIFY FOR CHANGES IN COST OR RHS, OR FOR ADDING OR DROPPING CONS.        
C
   40 CONTINUE
      CALL LPDVR2 (A,KT,RHS,COST,X,JX,PI,E,Y,DJ)
      GO TO 100
C
   50 CONTINUE
C
C     HERE, NEED TO CHANGE SIGN OF PI FOR MAX. PROBLEM.
C
      IF (MINMAX .EQ. 0) GO TO 70
      DO 60 J = 1, M1 
         PI(J) = -PI(J)
   60 CONTINUE
C
   70 CONTINUE 
      CALL LPDVR3 (A,KT,RHS,COST,X,JX,PI,E,Y,DJ)
      GO TO 100
C
C     FOR NEW PROBLEM, FIND A REASONABLE STARTING BASIS.
C
   80 CONTINUE
      CALL LPDPRE (A,KT,COST,JX)
C
C     FORM THE INVERSE OF THE BASIS.
C
   90 CONTINUE 
      CALL LPDVER (A,KT,RHS,COST,X,JX,PI,E,Y,DJ)
C
C     QUIT IF INFEASIBLE.
C
      IF (K .EQ. 2) GO TO 130
C
C     PERFORM DUAL ITERATIVE STEP.
C
  100 CONTINUE
      CALL LPDCYC (A,KT,RHS,COST,X,JX,PI,E,Y,DJ)
C
C     CHECK FOR INFEASIBILITY.
C
      IF (K .EQ. 2) IROW2 = IROW
      IF (K .GE. 1) GO TO 110
C
C     CHECK RE-INVERSION FREQUENCY.
C
      IF (INVC .GE. NVER) GO TO 90
C
C     ITERATE AGAIN.  
C
      GO TO 100
C
  110 CONTINUE
      IF (M1 .EQ. M .OR. K .GE. 2) GO TO 120
C
C     ARTIFICIAL CONSTRAINT WAS USED.
C     
      CALL LPDART (JX,E)
C
C     RE-INVERT TO REMOVE ARTIF. CONSTRAINT ONLY IF OPTIMAL.
C
      IF (K .EQ. 1) GO TO 90
C
C     IF K = 2 OR K = 3, RE-INVERT THE BASIS ONCE TO CHECK.
C
  120 CONTINUE
      IF (ITRY .EQ. 0 .OR. INVC .EQ. 0) GO TO 130
      ITRY = 0
C
      IF (IVER .EQ. 1 .OR. K .EQ. 2 .OR. K .EQ. 3 .OR.
     1   (K .EQ. 1 .AND. ITRYO .EQ. 1)) GO TO 90
C
C     COMPUTE ROW, COLUMN ERRORS, AND REDUCED COSTS AT SOLUTION.
C
  130 CONTINUE
      CALL LPDERC (A,KT,RHS,COST,X,JX,PI,E,DJ) 
C
C     RESTORE SIGN OF COSTS, CHANGE SIGN OF OBJECTIVE AND PI
C     FOR MAX. PROBLEM.
C
      IF (MINMAX .EQ. 0) GO TO 160
      DO 140 J = 1, NCOST
         COST(J) = -COST(J)
  140 CONTINUE
      DO 150 J = 1, M1
         PI(J) = -PI(J)
  150 CONTINUE
      OBVAL = -OBVAL 
C
  160 CONTINUE
C
C     IF OPTIMAL, RESET ANY INSIGNIFICANT X AND PI VALUES TO ZERO.
C 
      IF (K .NE. 1) GO TO 180
      DO 170 J = 1, M
         IF (DABS(X(J)) .LT. TX) X(J) = 0.0D0
         IF (DABS(PI(J)) .LT. TX) PI(J) = 0.0D0
  170 CONTINUE
C
  180 CONTINUE 
      RETURN
C
      END 



C
C-------------------------------------------------------------------------
C


      SUBROUTINE LPDMDF (A,KT,RHS,COST,X,JX,PI,E,Y,DJ)
C
C     MODIFIES PROBLEM AND TRANSFORMS E,X,PI WITHOUT
C     RE-INVERTING WHEN CONSTRAINTS ARE ADDED OR DROPPED.
C
      INTEGER  KT,JX
      REAL  A,RHS,COST
      DOUBLE PRECISION  X,PI,E,Y,DJ
      DIMENSION  A(*),KT(*),RHS(*),COST(*),PI(*),X(*),JX(*),
     1   E(*),Y(*),DJ(*)  
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      INTEGER  I,ID,IOP,J,J4,JJ,JM,JMM,KK,MFLAG,MM,NN
      DOUBLE PRECISION  A4,TEMP 
C
C
      IF (IDROP .GT. 0) GO TO 70
C  
C     ENTER HERE FOR ADDITION OF CONSTRAINTS.
C  
C     UPDATE DIAGONAL OF E. 
   10 CONTINUE
      M = M - IADD  
      N = N - IADD  
      MM = M*MLE + M
      NN = N*ML + M 
      DO 20 I = 1, IADD 
         E(MM+I) = A(NN+I)
         NN = NN + ML  
         MM = MM + MLE 
   20 CONTINUE
C
C     COMPUTE NEW ROWS IN E, ONE ROW AT A TIME.  NEW ROW = -A4*C*E.
C  
      DO 60 ID = 1, IADD
         DO 30 I = 1, M
            J = ML*(JX(I)-1) + M + 1
            Y(I) = A(J)
   30    CONTINUE 
         JM = M*MLE + M + 1  
         A4 = E(JM)  
         JM = 0  
         DO 50 I = 1,M
            JJ = JM + M + 1 
            E(JJ) = 0.0D0
            DO 40 J = 1, M
               JMM = JM + J  
               IF (DABS(E(JMM)) .LT. TX) GO TO 40
               E(JJ) = E(JJ) - Y(J)*E(JMM)*A4
   40       CONTINUE  
            JM = JM + MLE
   50    CONTINUE 
         M = M + 1 
         N = N + 1 
   60 CONTINUE  
      GO TO 160 
C  
C     ENTER HERE TO DROP A CONSTRAINT.  
C  
   70 CONTINUE 
      MFLAG = 0
      M = M + 1 
      IOP = IDROP 
   80 CONTINUE
      MM = IOP
      NN = M  
C
C     EXCHANGE ROWS IDROP AND M IN E-MATRIX.
C
      DO 90 I = 1, M
         A4 = E(MM)  
         E(MM) = E(NN)
         E(NN) = A4  
         MM = MM + MLE 
         NN = NN + MLE 
   90 CONTINUE
C
C     EXCHANGE IN JX.
C
      J4 = JX(IOP)
      JX(IOP) = JX(M) 
      JX(M) = J4  
      IF (MFLAG .EQ. 1) GO TO 120
C
C     EXCHANGE IN RHS.
C
      A4 = RHS(IOP)
      RHS(IOP) = RHS(M)
      RHS(M) = A4 
C
C     A-MATRIX NEEDS ROW M PLACED IN ROW IDROP-----NO EXCHANGE. 
C     ALSO REMOVE THE DROPPED ROW.  
C
      MM = IOP
      NN = M  
      DO 100 I = 1, N1  
         A(MM) = A(NN)
         A(NN) = 0.0  
         MM = MM + ML  
         NN = NN + ML  
  100 CONTINUE
C
C     EXCHANGE COLUMNS IDROP AND M IN E-MATRIX.
C
      MM = (IOP-1)*MLE
      NN = (M-1)*MLE  
      DO 110 I = 1, M
         MM = MM + 1
         NN = NN + 1
         A4 = E(MM)  
         E(MM) = E(NN)
         E(NN) = A4  
  110 CONTINUE
      IF (DABS(A4) .LT. 1.E-4) GO TO 170
  120 CONTINUE 
      M = M - 1
C 
C     COMPUTE NEW INVERSE E = B1 - B2*B3/B4.
C 
      DO 140 I = 1, M
         DO 130 J = 1, M
         MM = (J-1)*MLE + I
         NN = M*MLE + I
         KK = (J-1)*MLE + M + 1  
         E(MM) = E(MM)-E(NN)*E(KK)/A4
  130    CONTINUE
  140 CONTINUE
C
C     CLEAN UP RHS,JX,X, AND OLD ROW AND COLUMN IN E.
C
      NN = M + 1
      RHS(NN) = 0.0
      X(NN) = 0.0D0
      JX(NN) = 0  
      KK = M*MLE + 1
      DO 150 I = 1, M
         E(NN) = 0.0D0
         NN = NN + MLE 
         E(KK) = 0.0D0  
         KK = KK + 1
  150 CONTINUE
      E(KK) = 0.0D0
  160 CONTINUE 
      M1 = M  
      N1 = N  
      GO TO 200
C 
C     AT THIS POINT WE HAVE A ZERO PIVOT ELEMENT B4, SO WE MUST SELECT  
C     A NEW ROW FOR EXCHANGE.  FIND I S.T. ABS(E(I,M)) IS MAXIMIZED. 
C
  170 CONTINUE 
      MFLAG = 1
      A4 = 1.E-4  
      MM = (M-1)*MLE  
      KK = M-1
      IOP = 0 
      DO 180 I = 1, KK  
         MM = MM + 1
         TEMP = DABS(E(MM))  
         IF (TEMP .LE. A4) GO TO 180
         A4 = TEMP
         IOP = I 
  180 CONTINUE  
C
C     FOUND POSSIBLE PIVOT ELEMENT.  GO BACK AND EXCHANGE ROWS IN E, JX.
      IF (IOP .NE. 0) GO TO 80
C
C     BAD NEWS.
C 
      WRITE (IO1,190)
      IF (IO3 .NE. IO1) WRITE (IO3,190)  
  190 FORMAT (/1X,'ALGORITHM ERROR IN LPDMDF-ROUTINE.' 
     1  '  CANNOT PIVOT ON ZERO.'/)
      K = 7
C
  200 CONTINUE  
C 
      RETURN
      END



      SUBROUTINE LPDVER (A,KT,RHS,COST,X,JX,PI,E,Y,DJ)
C
C     FORMS THE INVERSE  E OF SIZE M1 X M1.
C     THE DJ ARRAY IS USED AS A BASIC COLUMN INDICATOR, AND
C     LATER IN LPDVR2 TO HOLD THE ARTIFICAL CONSTRAINT. 
C     IT IS ALSO USED FOR DJ IN LPDUPC AND LPDDJD WITHOUT CONFLICT. 
C
      DIMENSION A(*),KT(*),RHS(*),COST(*),X(*),JX(*),PI(*),E(*),Y(*),
     1   DJ(*)
      INTEGER  KT,JX
      REAL  A,RHS,COST
      DOUBLE PRECISION  X,PI,E,Y,DJ
C
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C
      INTEGER  I,II,IK,IL,INDX,J,JDUL,L,LL,M2,MM,NN
      DOUBLE PRECISION  C,COM
C
C     CONSTANT.
C
      DATA  C / 1000.0D0 /
C
C
C     STAGE 1 - MAKE DUAL FEASIBLE, IF REQUIRED.
C
C     STAGE 2 - DRIVE OUT ARTIFICALS, IF REQUIRED. 
C
C     STAGE 3 - MAKE PRIMAL FEASIBLE I.E. OPTIMIZE THE LINEAR OBJ. FORM.
C
C     THE M + 1 COLUMN OF E IS USED AS A PIVOT LIST. 
C
C     ZERO OUT DJ AND INITIALIZE E.
C   
      LL = M*MLE  
      NN = N + M*IALLEQ
      DO 100 J = 1, NN  
         DJ(J) = 0.0D0
  100 CONTINUE
      INVC = 0
      NUMVR = NUMVR + 1
      M2 = M + 1
      MM = 0  
      DO 180 II = 1, M2 
         DO 150 I = 1, M2  
            E(I+MM) = 0.0D0
  150    CONTINUE  
         MM = MM + MLE 
  180 CONTINUE
C
C     PUT E INTO M+1 X M+1 IDENTITY FORM.
C
      MM = 0  
      DO 200 I = 1, M2 
         E(I+MM) = 1.0D0
         MM = MM + MLE 
  200 CONTINUE
C
C     DETERMINE IF M1 = M, REMOVE N + M + 1 FROM JX AND RESET M1 = M.
C
      IF (M1 .EQ. M) GO TO 300  
      DO 250 I = 1, M1  
         IF (JX(I) .NE. N1) GO TO 250  
         JX(I) = JX(M1)  
         GO TO 260
  250 CONTINUE  
  260 CONTINUE
      M1 = M  
      N1 = N + M*IALLEQ
C
C     PIVOT AS MANY JX COLUMNS AS POSSIBLE.
C     SET IPIV = 1, FOR PIVOT ON E ONLY.
C     OBVAL, PI, AND X ARE COMPUTED LATER IN ENTRY PTS LPDVR2 AND LPDVR3.
C
  300 CONTINUE
      IPIV = 1
      DO 600 I = 1, M
         JCOL = JX(I)
         IF (JCOL .EQ. 0) GO TO 600 
         CALL LPDUPC (A,KT,E,Y,DJ)  
C
C        THE UPDATED COLUMN IS IN Y.  SET TO ZERO SMALL NUMBERS IN Y.
C
         DO 350 II = 1, M  
            IF (DABS(Y(II)) .GT. TX) GO TO 350
            Y(II) = 0.0D0
  350    CONTINUE  
C
C        FIND LARGEST ABS(Y(IK)) ON UNUSED PIVOT LOCATIONS.
C        THE M + 1 COL. OF E IS USED AS A PIVOT OCCUPANCY LIST.  
C
         INDX = 0
         COM = TPIV  
         DO 400 IK = 1, M  
            IF (E(IK+LL) .GE. C) GO TO 400
            IF (DABS(Y(IK)) .LT. COM) GO TO 400
            INDX = IK  
            COM = DABS(Y(IK))
  400    CONTINUE  
         IF (INDX .EQ. 0 .OR. COM .LE. TPIV) GO TO 600
         IROW = INDX
         CALL LPDPIV (X,PI,E,Y) 
         DJ(JCOL) = INDX
         E(INDX+LL) = C 
  600 CONTINUE  
C
C     DETERMINE THE JX VECTOR FROM DJ, RESET M+1 COL OF E, SET IPIV = 0.      
C
      IPIV = 0
      DO 650 I = 1, M
         E(I+LL) = 0.0D0
         JX(I) = 0
  650 CONTINUE
      DO 660 J = 1, N1  
         IL = DJ(J)  
         IF (IL .GT. 0) JX(IL) = J
         DJ(J) = 0
  660 CONTINUE
      IF (K .EQ. 2) RETURN 
C
C     HAVE A FULL BASIS AT THIS POINT.  
C
C.......................................................................
C
      ENTRY LPDVR2 (A,KT,RHS,COST,X,JX,PI,E,Y,DJ)
C
C     ENTRY LPDVR2 IS USED CHECK FOR DUAL FEAS., IF NOT, WE INSTALL
C     ARTIFICAL CONSTRAINT AND WE CALCULATE X, PI, AND OBVAL.
C
C     THIS ENTRY POINT AND LPDVR3 ARE CALLED BY LPDMDF.  
C
C     PUT C-B IN Y, SO WE MAY CALCULATE THE PI VECTOR, USING LPDCBV.
C
      DO 700 I = 1, M
         Y(I) = 0.0D0  
         L = JX(I)
         IF (L .EQ. 0. OR. (L .GT. N .AND. KSLACK .EQ. 0)) GO TO 700
         Y(I) = COST(L)  
  700 CONTINUE  
      PI(M+1) = 0.0D0
      CALL LPDCBV (PI,E,Y)
C
C     LOOK FOR NEGATIVE DJS OVER NON-BASIC. SET CORR. DJ(I) = 1. 
C     FIND MOST NEGATIVE DJ, SAVE COL. INDX IN JDUL.
C
      COM = 0.0D0
      JDUL = 0
      DO 800 J = 1, N1  
         DJ(J) = 0.0D0  
         DO 750 I = 1, M
         IF (JX(I) .EQ. J) GO TO 800
  750    CONTINUE  
         JCOL = J
         CALL LPDDJD (A,KT,COST,PI,E,DJ)
         IF (DJ1 .GE. -TCOST) GO TO 800
         DJ(J) = 1.0D0
         IF (DJ1 .GT. COM) GO TO 800
         COM = DJ1
         JDUL = J
  800 CONTINUE  
      DJ1 = COM
      X(M+1) = TART
      IF (K .GE. 1) X(M+1) = 10.0*TART  
      K = 0
C
C.......................................................................
C
      ENTRY LPDVR3 (A,KT,RHS,COST,X,JX,PI,E,Y,DJ) 
C
C     ENTRY LPDVR3 CALCULATES X AND OBVAL.
C     FIRST WE CALCULATE OBVAL USING PI AND RHS.
C
      OBVAL = 0.0D0
      DO 850 I = 1, M
         IF (DABS(PI(I)) .LE. TX) GO TO 850 
         IF (RHS(I) .EQ. 0.0) GO TO 850
         OBVAL = OBVAL + PI(I)*RHS(I)  
  850 CONTINUE  
      OBVAL = -OBVAL  
C
C     SKIP COMPUTING X IF COST ONLY CHANGE PROBLEM I.E. INFLAG = 4.
C
      IF (INFLAG .EQ. 4 .AND. NUMVR .EQ. 0) GO TO 960
C
C     PREPARE TO USE LPDUPC.  STORE COL.1 OF A IN X.  FILL COL.1 W. RHS.      
C
      DO 900 I = 1, M
         X(I) = A(I) 
         A(I) = RHS(I)
  900 CONTINUE
      JCOL = 1
      CALL LPDUPC (A,KT,E,Y,DJ)  
C
C     RESET EVERYTHING. 
C
      DO 950 I = 1, M
         A(I) = X(I) 
         X(I) = Y(I) 
  950 CONTINUE
  960 CONTINUE
      IF (INFLAG .EQ. 3 .AND. NUMVR .EQ. 0) GO TO 990  
      IF (JDUL .EQ. 0) GO TO 990
      JCOL = JDUL 
      M1 = M + 1
      N1 = N + M*IALLEQ + 1
      JX(M1) = JCOL
      IROW = M1
      CALL LPDUPC (A,KT,E,Y,DJ)  
      CALL LPDPIV (X,PI,E,Y) 
C
  990 CONTINUE
      RETURN
C
      END


C
C-------------------------------------------------------------------------
C


      SUBROUTINE LPDUPC (A,KT,E,Y,DJ)
C
C     THIS ROUTINE WILL UPDATE COL JCOL AND STORE THE UPDATED COL IN Y.
C     CALCULATES Y = E * JCOL OF A.
C
      INTEGER  KT
      REAL  A
      DOUBLE PRECISION  E,Y,DJ
      DIMENSION A(*),KT(*),E(*),Y(*),DJ(*)
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      INTEGER  I,I1,JJ,MM,MMM
C
C
      DO 10 I = 1, M1
         Y(I) = 0.0D0
   10 CONTINUE
      IF (JCOL .GT. N) GO TO 150
      MM = (JCOL-1)*ML  
      DO 120 I = 1, M1
         MMM = 0 
         DO 110 JJ = 1, M
            IF (A(JJ+MM) .EQ. 0.0) GO TO 100 
            IF (DABS(E(I+MMM)) .LE. TX) GO TO 100 
            Y(I) = Y(I) + A(JJ+MM)*E(I+MMM)
  100       CONTINUE 
            MMM = MMM + MLE
  110    CONTINUE
         IF (M1 .EQ. M) GO TO 120
         IF (DABS(E(I+MLE*M)) .LE. TX) GO TO 120
         IF (DJ(JCOL) .LE. 0.0D0) GO TO 120  
         Y(I) = Y(I) + E(I+MLE*M)
  120 CONTINUE  
      GO TO 400
C
C     IF JCOL IS THE ART. SLACK,THE UP-DATED COL. IS THE M+1 COL. OF E. 
C
  150 CONTINUE
      I1 = JCOL - N 
      IF (I1 .EQ. (M*IALLEQ+1)) GO TO 300  
      IF (KT(I1) .EQ. 0) GO TO 400
      MM = (I1-1)*MLE
      DO 200 I = 1, M1
         IF (DABS(E(I+MM)) .LE. TX) GO TO 200
         Y(I) = E(I+MM)*DFLOAT(KT(I1))
  200 CONTINUE  
      IF (M1 .EQ. M) GO TO 400
      IF (DABS(DJ(JCOL)) .LE. 0.0D0) GO TO 400  
      MMM = M * MLE  
      DO 210 I = 1, M1 
         Y(I)= Y(I) + E(MMM+I)
  210 CONTINUE 
      GO TO 400
C
  300 CONTINUE
      MM = M*MLE  
      DO 350 I = 1, M1 
         Y(I) = E(I+MM)  
  350 CONTINUE
C
  400 CONTINUE
      RETURN
C
      END

C
C-------------------------------------------------------------------------
C


      SUBROUTINE LPDPIV (X,PI,E,Y)
C
C     TRANSFORMS OBVAL, X, PI, AND E BY PIVOTING, I.E., USING THE 
C     TRANSFORMATION UNEW = UOLD + UR*PHI WHERE U IS A COLUMN VECTOR AND
C     PHI IS THE COLUMN VECTOR PHI =(-Y1/YR,...1/YR -1,...YM/YR).
C
      DOUBLE PRECISION  X,PI,E,Y 
      DIMENSION  X(*),PI(*),E(*),Y(*)
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C 
      INTEGER  I,II,III,J,MM
      DOUBLE PRECISION  DELTAJ,EE,Z
C
C
C     TRANSFORM E AND PI.  NOTE IROW IS THE PIVOT INDEX. 
C     IPIV IS A PIVOT ON E ONLY FLAG.
C     NOTE DELTA-J FOR JCOL IS CONTAINED IN VAR. DJ1
C
      MM = 0  
      Z = Y(IROW) 
      DELTAJ = DJ1
      Y(IROW) = -1.0D0  
      DO 120 J = 1, M1 
         III = IROW + MM
         IF (DABS(E(III)) .LE. TX) GO TO 110  
         EE = E(III)/Z
         E(III) = 0.0D0
         DO 100 I = 1, M1  
            II = I + MM
            IF (DABS(Y(I)) .LE. TX) GO TO 100 
            E(II) = E(II) - Y(I)*EE
  100    CONTINUE  
         IF (IPIV .EQ. 1) GO TO 110
         PI(J) = PI(J) - DELTAJ*EE 
  110    CONTINUE
         MM = MM + MLE 
  120 CONTINUE
C
C     IPIV = 1 MEANS PIVOTING ONLY ON E IN LPDVER.
C
      IF (IPIV .EQ. 1) GO TO 300
C 
C     TRANSFORM X AND OBVAL.  
C
      IF (DABS(X(IROW)) .LE. TX) GO TO 210 
      EE = X(IROW)/Z  
      X(IROW) = 0.0D0
      DO 200 I = 1, M1 
         IF (DABS(Y(I)) .LE. TX) GO TO 200
         X(I) = X(I) - Y(I)*EE 
  200 CONTINUE  
      OBVAL = OBVAL + DELTAJ*EE
  210 CONTINUE
      Y(IROW) = Z
C
  300 CONTINUE 
      RETURN
C
      END



C
C-------------------------------------------------------------------------
C


      SUBROUTINE LPDCBV (PI,E,Y) 
C
C     CALCULATES PI = -Y TIMES B INVERSE WHERE Y IS USUALLY SET TO C-B.
C
      DOUBLE PRECISION  PI,E,Y
      DIMENSION  PI(*),E(*),Y(*) 
C
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      INTEGER  I,J,JJ,JM
C
C
      JM = 0  
      DO 100 I = 1, M1 
         PI(I) = 0.0 
         DO 50 J = 1,M1  
            JJ = J + JM
            IF (DABS(E(JJ)) .LE. TX) GO TO 50
            IF (DABS(Y(J)) .LE. TX) GO TO 50 
            PI(I) = PI(I) + Y(J)*E(JJ)
   50    CONTINUE  
         JM = JM + MLE 
         PI(I) = -PI(I)  
  100 CONTINUE  
C
      RETURN
      END


C
C-------------------------------------------------------------------------
C


      SUBROUTINE LPDDJD (A,KT,COST,PI,E,DJ)
C
C     LPDDJD COMPUTES DJ1 OR REDUCED COST FOR COL. JCOL  
C     DJ1 = COST(JCOL) + PI*JCOL OF A.
C
      INTEGER  KT
      REAL  A,COST
      DOUBLE PRECISION  PI,E,DJ
      DIMENSION  A(*),KT(*),COST(*),PI(*),E(*),DJ(*)  
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C
      INTEGER  I,II,III,J,MM,MMM
C
C
      J = JCOL
      DJ1 = 0.0D0
      IF (J .GT. N) GO TO 200 
      MM = (J-1)*ML  
C
      DO 100 I = 1, M  
         II = I + MM
         IF (ABS(A(II)) .LE. TX) GO TO 100
         IF (DABS(PI(I)) .LE. TX) GO TO 100
         DJ1 = DJ1 + PI(I)*A(II)
  100 CONTINUE  
C
      DJ1 = DJ1 + COST(J)
      IF (M1 .EQ. (M+1)) DJ1 = DJ1 + PI(M+1)*DJ(J)
      GO TO 300
C
  200 CONTINUE
      I = J - N 
      DJ1 = PI(M+1)
      IF (I .GT. (M*IALLEQ)) GO TO 300
      DJ1 = 0.0D0
      IF (KT(I) .EQ. 0) GO TO 300
      DJ1 = PI(I)*DFLOAT(KT(I)) + PI(M+1)*DJ(J)
      IF (KSLACK .EQ. 1) DJ1 = DJ1 + COST(J)
C
  300 CONTINUE
      RETURN
C
C.......................................................................
C
      ENTRY LPDYJD (A,KT,E,DJ)
C
C     COMPUTES YJ1 FOR JCOL AT THE IROW LEVEL USING  
C     YJ1 = IROW OF E * JCOL OF A.
C
      J = JCOL  
      YJ1 = 0.0D0  
      IF (J .GT. N) GO TO 600
C
      MM = (J-1)*ML 
      MMM = 0 
C
      DO 500 I = 1, M  
         III = MMM + IROW  
         IF (DABS(E(III)) .LE. TX) GO TO 400
         IF (A(I+MM) .EQ. 0.0) GO TO 400
         YJ1 = YJ1 + E(III)*A(I+MM)  
  400    CONTINUE
         MMM = MMM + MLE
  500 CONTINUE
C
      IF (M1 .EQ. (M+1)) YJ1 = YJ1 + E(MLE*M+IROW)*DJ(J)
      GO TO 800
C
  600 CONTINUE
      I = J - N 
      IF (I .GT. (M*IALLEQ)) GO TO 700 
      IF (KT(I) .EQ. 0) GO TO 800
      II = I - 1
      YJ1 = E(MLE*II+IROW)*DFLOAT(KT(I)) + E(MLE*M+IROW)*DJ(J) 
      GO TO 800
C
  700 CONTINUE
      YJ1 = E(MLE*M+IROW)
C
  800 CONTINUE
C
      RETURN
      END


C
C-------------------------------------------------------------------------
C

      SUBROUTINE LPDART (JX,E)
C
C     CHECKS FOR POSITION AND PRESENCE OF THE ARTIFICAL SLACK IN THE
C     BASIS.  SETS IVER OR K = 3.  NOTE THIS SUBROUTINE IS CALLED ONLY
C     IF THE ARTIFICAL CONSTRAINT WAS USED,THUS N1 = N + M + 1.  
C
      INTEGER  JX
      DOUBLE PRECISION  E
      DIMENSION  JX(*),E(*)  
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      INTEGER  I,J,JJ,LL
C
C
C     CHECK IF ART. SLACK IS IN THE BASIS. 
C     IF IN THE BASIS AND NOT AT THE END SET IVER = 1.
C
      DO 50 I = 1, M
         IF (JX(I) .EQ. N1) GO TO 100 
  50  CONTINUE  
C
C     IF ARTIFICAL SLACK IS IN THE BASIS AND AT THE END.ZERO OUT THE
C     LAST ROW AND COLUMN OF E AND RESET M1 AND N1  
C
C.......................................................................
C
      ENTRY LPDAR2 (JX,E)
C
C     LPDAR2 CHECKS IF ART. SLACK IS IN FINAL POSITION IN BASIS. 
C
      IF (JX(M1) .EQ. N1) GO TO 200
C
C     ARTIFICAL SLACK NOT IN THE BASIS.  SET K = 3 (UNBOUNDED SOL.)  
C
      K = 3
      GO TO 400
C
  100 CONTINUE 
      IVER = 1
      GO TO 400
  200 CONTINUE 
      LL =  M*MLE 
      DO 250 I = 1, M1 
         E(I+LL) = 0.0D0
  250 CONTINUE
      DO 300 J = 1,M1 
         JJ = J - 1
         E(M1+JJ*MLE) = 0.0D0
  300 CONTINUE
      E(M1+LL) = 1.0D0
C
C     RESET M1 AND N1
C
      M1 = M  
      N1 = N + M*IALLEQ
C
  400 CONTINUE
      RETURN
C
      END

C
C-------------------------------------------------------------------------
C



      SUBROUTINE LPDCOL (A,KT,COST,X,JX,PI,E,DJ) 
C
C     COMPUTES THE INDEX,INCOL,OF THE INCOMING COLUMN BY CALLING LPDMIN.
C
      INTEGER  KT,JX
      REAL  A,COST
      DOUBLE PRECISION  X,PI,E,DJ
      DIMENSION  A(*),KT(*),COST(*),X(*),JX(*),PI(*),E(*),DJ(*)
C
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C
      DOUBLE PRECISION  XX
C
C
C     SET EP = 1.0, HOWEVER IF X NON-NEGATIVE AND ARTIFICAL,
C     SET EP = -1.0  IN ORDER TO CHOOSE A POSITIVE YJ1. 
C
      EP = 1.0D0
      XX =  X(IROW)
C
C     NOTE IN STAGE 3 XX WILL BE NEGATIVE AND JX(IROW) NON-ART.
C
      IF (JX(IROW) .EQ. 0 .AND. XX .GE. 0.0D0) EP = -1.0
      CALL LPDMIN (A,KT,COST,JX,PI,E,DJ) 
      IF (INCOL .GT. 0) GO TO 100 
C
C     K = 2 MEANS NO FEASIBLE SOLUTION.
C
      K = 2
      IF (JX(IROW) .GT. 0) GO TO 100  
C
C     THE NEXT CALL TO LPDMIN MAY ONLY OCCUR WHEN ART.S ARE IN  
C     THE BASIS AT NON ZERO LEVEL.  CHANGE SIGN OF EP.
C
      EP = -EP
      CALL LPDMIN (A,KT,COST,JX,PI,E,DJ)
      IF (INCOL .EQ. 0) GO TO 100 
      K = 0
C
  100 CONTINUE
      RETURN
C
      END

C
C-------------------------------------------------------------------------
C

      SUBROUTINE LPDCYC (A,KT,RHS,COST,X,JX,PI,E,Y,DJ)
C
C     PERFORMS THE DUAL SIMPLEX ITERATIVE STEP.
C
      INTEGER  KT,JX
      REAL  A,RHS,COST
      DOUBLE PRECISION  X,PI,E,Y,DJ
      DIMENSION  A(*),KT(*),RHS(*),COST(*),X(*),JX(*),PI(*),E(*),Y(*),
     1   DJ(*)  
C
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C
C
C     CHECK ITERATION LIMIT 
      K = 4
      IF (ITER .GE. NCUT) GO TO 100
      K = 0
C
C
C     CHECK WHETHER THE ARTIFICAL CONSTRAINT MAY BE DELETED 
      IF (M1 .EQ. (M+1)) CALL LPDAR2 (JX,E) 
      K = 0
C
C     DETERMINE THE PIVOT ROW, IROW.  K = 1 IS OPT. SOL. 
C
      CALL LPDNEG (X,JX) 
      IF (K .EQ. 1) GO TO 100  
C
C     FIND THE ENTERING COLUMN INDEX, INCOL.  K = 2 IS INFEAS. SOLUTION.  
C
      CALL LPDCOL (A,KT,COST,X,JX,PI,E,DJ)
      IF (K .EQ. 2) GO TO 100  
C
C     PREPARE TO PIVOT, CALL LPDDJD TO GET DJ.
C
      JCOL = INCOL
      CALL LPDDJD (A,KT,COST,PI,E,DJ)  
C
C     UPDATE INCOL AND RETURN UPDATED COLUMN TO Y. 
C
      CALL LPDUPC (A,KT,E,Y,DJ)  
      OB = OBVAL
C
C     PIVOT, CHANGING THE BASIS. 
C
      CALL LPDPIV (X,PI,E,Y) 
C
C     UPDATE COUNTERS, ETC.  K = 4 MEANS ITERATION LIMIT REACHED.
C
      JOUT = JX(IROW) 
      JX(IROW) = INCOL
      ITER = ITER + 1
      INVC = INVC + 1
C
C     K = 5  MEANS EQUAL OBVALECTIVE LIMIT REACHED. 
C
      IF (DABS(OBVAL-OB) .GE. TX) NEQL = -1
      NEQL = NEQL + 1
      K = 5
      IF (NEQL .GE. NEQ) GO TO 100 
      K = 0
C
  100 CONTINUE
C
C     OUTPUT LINE OF INTERMEDIATE OUTPUT IF REQUESTED.
C
      IF (ILINE .EQ. 1) CALL LPDW31 
C 
      RETURN
      END


C
C-------------------------------------------------------------------------
C

      SUBROUTINE LPDERC (A,KT,RHS,COST,X,JX,PI,E,DJ)
C
C     COMPUTES ERRORS AND REDUCED COSTS. 
C
      INTEGER  KT,JX
      REAL  A,RHS,COST
      DOUBLE PRECISION  X,PI,E,DJ
      DIMENSION  A(*),KT(*),RHS(*),COST(*),X(*),JX(*),PI(*),E(*),DJ(*)
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C
      INTEGER  I,II,J,JSS
      REAL  ERROR
      DOUBLE PRECISION  SS
C
C
C     COMPUTE SUM OF ABSOLUTE VALUES OF A*Z - RHS.
C
      DO 200 II = 1, M 
         DJ(II) = -RHS(II)
         SS = DJ(II)  
         DO 100 I = 1, M  
            IF (JX(I) .EQ. 0) GO TO 100 
            JSS = JX(I) - N  
            IF (JSS .GT. 0) GO TO 90
            J = II + (JX(I)-1)*ML 
            IF (A(J) .EQ. 0.0) GO TO 100
            IF (X(I) .EQ. 0.0D0) GO TO 100
            SS = SS + A(J)*X(I)
            GO TO 100
   90       CONTINUE
            J = JX(I) - N
            IF (J .EQ. II) SS = SS + DFLOAT(KT(J))*X(I)  
  100    CONTINUE
         DJ(II) = SS
         ERROR = DABS(SS)
         ROWSUM = ROWSUM + ERROR  
         IF (ERROR .GE. ROWMAX) ROWMAX = ERROR
  200 CONTINUE  
C
C     FIND SUM AND MAXIMUM OF REDUCED COSTS IN BASIS.
C
  250 CONTINUE
      DO 260 J = 1, N1 
         JCOL = J
         CALL LPDDJD (A,KT,COST,PI,E,DJ)  
         DJ(J) = DJ1
  260 CONTINUE
      DO 300 I = 1, M  
         J = JX(I) 
         IF (J .EQ. 0) GO TO 300  
         DJ1 = DJ(J)
         ERROR = DABS(DJ1)  
         COLSUM = COLSUM + ERROR  
         IF (ERROR .GT. COLMAX) COLMAX = ERROR
  300 CONTINUE  
C
      RETURN
      END

C
C-------------------------------------------------------------------------
C

      SUBROUTINE LPDMIN (A,KT,COST,JX,PI,E,DJ)
C
C     COMPUTES THE MINIMUM RATIO, RAT, OF DJ1/YIJ OVER YIJ*EP NEGATIVE,  
C     WITH I EQUAL IROW, J NON-BASIC, EP EQUAL -1 OR 1.  ALSO THE INDEX 
C     CORRESPONDING TO THE MINIMUM RATIO,RAT,IS SET IN INCOL.
C     THIS SUBROUTINE IS CALLED BY LPDCOL WHICH DETERMINES THE IN-COMING
C     COLUMN.  TIES ARE BROKEN 'RANDOMLY'.
C
      INTEGER  KT,JX
      REAL  A,COST
      DOUBLE PRECISION  PI,E,DJ
C
      DIMENSION  A(*),KT(*),COST(*),JX(*),PI(*),E(*),DJ(*)
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C
      INTEGER  I,ITIE,J
      REAL  R,SS
      DOUBLE PRECISION  COM,YJ2,DJ2,ZSEED,ZMULT,ZMOD,ZSCALE  
C
C     CONSTANTS FOR RANDOM NO. GENERATION
      SAVE ZSEED
      DATA ZSEED /123457.D0/,   ZMOD/2147483647.D0/,
     1     ZMULT / 16807.D0/, ZSCALE/2147483648.D0/ 
C
C
C     LOOK OVER NON-BASIC COLUMNS.
C
      RAT = 0.0D0
      COM =  1.0E+20
      ITIE = 1
      INCOL = 0 
      DO 100 J = 1, N1 
         DO 20  I = 1, M1 
         IF (JX(I) .EQ. J) GO TO 100  
   20    CONTINUE  
         IF (J .LE. N. OR. J .GT. (N+M)) GO TO 30 
         IF (J .EQ. N1 .AND. M1 .EQ. (M+1)) GO TO 30  
C
C        SKIP SLACK COLUMNS WHEN EQUALITIES........T(I) = 0.  
C
         IF (KT(J-N) .EQ. 0) GO TO 100 
   30    CONTINUE  
C
C        PREPARE TO CALL LPDYJD IN ORDER TO COMPUTE YJ1. 
C
         JCOL = J
         CALL LPDYJD (A,KT,E,DJ)
         IF (DABS(YJ1) .LE. TPIV) GO TO 100
         YJ2 = EP*YJ1
         IF (YJ2 .GE. 0.0D0) GO TO 100
         YJ2 = DABS(YJ2) 
C
C        COMPUTE THE CORRESPONDING DJ1. 
C
         CALL LPDDJD (A,KT,COST,PI,E,DJ)  
         DJ2 = DABS(DJ1) 
         RAT = DJ2/YJ2
         IF (DJ2 .LE. TX) RAT = 0.0D0
         IF (DJ2 .LE. TX) DJ2 = 0.0D0  
         IF (RAT - COM) 70, 50, 100 
   50    CONTINUE  
         ITIE = ITIE + 1  
         SS = ITIE
         SS = 1.0/SS 
         ZSEED =  DMOD(ZMULT*ZSEED, ZMOD)
         R = ZSEED/ZSCALE
         IF (R .LE. SS) INCOL = J
         GO TO 100 
   70    CONTINUE
         COM = RAT
         INCOL = J
         ITIE = 1
  100 CONTINUE  
      RAT = COM
C
      RETURN
      END


C
C-------------------------------------------------------------------------
C

      SUBROUTINE LPDNEG (X,JX)
C
C     FINDS ROW INDEX, IROW, WITH LARGEST IN ABS. VALUE OVER ARTIFICALS. 
C     IF ARTS ARE ZERO, FINDS THE INDEX WITH MOST NEGATIVE X.
C     NOTE IF MAX/X(I)/ OVER ARTS LESS THAN TFEAS/M,THEN
C     WE TAKE ARTS TO BE AT ZERO LEVEL OR FEASIBLY ADMISSABLE.  
C
      INTEGER  JX
      DOUBLE PRECISION  X
      DIMENSION  X(*),JX(*)  
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C
      INTEGER  I
      REAL  SM
      DOUBLE PRECISION  COM,XX
C
C
C     CHOOSE LARGEST CORRESPONDING ART. (IN ABS. VALUE).
C
      SM = M  
      IROW = 0
      COM =  TFEAS/SM 
      DO 50 I = 1, M1  
         IF (JX(I) .GT. 0) GO TO 50
         XX = DABS(X(I)) 
         IF (XX .LT. COM) GO TO 50
         IROW = I
         COM = XX
  50  CONTINUE  
      XMOST = COM 
C
C     CHECK IROW, IF NOT ZERO RETURN. 
C
      IF (IROW .NE. 0) GO TO 200
C
C     CHOOSE THE MOST NEGATIVE X OVER NON-ARTS. 
C
  100 CONTINUE
      COM = -TFEAS/SM
      DO 150 I = 1, M1 
         IF (JX(I) .EQ. 0) GO TO 150  
         XX = X(I)
         IF (XX .GE. COM) GO TO 150  
         IROW = I
         COM = XX
  150 CONTINUE  
      XMOST = COM 
      IF (IROW .EQ. 0) K = 1 
C
  200 CONTINUE
      RETURN
C
      END


C
C-------------------------------------------------------------------------
C

      SUBROUTINE LPDPRE (A,KT,COST,JX)  
C
C     FLAGS A REASONABLE STARTING BASIS, FOR INFLAG = 0 PROBLEMS. 
C
      INTEGER  KT,JX
      REAL  A,COST
      DIMENSION  A(*),KT(*),COST(*),JX(*)
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      INTEGER  I,ICNT,II,INDX,IW,IW2,MM,NNZ 
C
C
C     INITIALIZE JX 
C
C     PUT SLACKS WITH NEGATIVE COSTS IN THE BASIS FIRST.
C
      DO 10 I = 1, M
         JX(I) = 0
   10 CONTINUE
      IF (KSLACK .EQ. 0) GO TO 30
      IW = N + 1  
      IW2 = N + M  
      DO 20 I = IW, IW2  
         J = I - M
         IF (COST(I) .LT. 0.0 .AND. KT(J) .NE. 0) JX(J) = I   
   20 CONTINUE  
C
C     SEARCH NEGATIVE COST FOR REAL VARIABLES
C
   30 CONTINUE  
      DO 50 I = 1, N
      IF (COST(I) .GE. 0.0)  GO TO  50
C
      DO 40 J = 1, M 
         IF (JX(J) .GT. 0) GO TO 40
         II = (I-1)*ML
         IF (A(II+J) .EQ. 0.0) GO TO 40  
         JX(J) = I
         GO TO 50  
   40 CONTINUE  
C
C
C     SEARCH FOR SINGLETONS AMONG THE REAL COLS REGARDLESS OF THE COST.        
C
   50 CONTINUE  
      NNZ = 0 
      MM = 0 
      INDX = 0 
      DO 100 J = 1, N
C
C        COUNT THE NON-ZERO ENTRIES IN COL J.
C
         ICNT = 0 
         DO 70 I = 1, M
            II = I + MM
            IF (A(II) .EQ. 0.0) GO TO 70 
            ICNT = ICNT + 1
            INDX = I
   70    CONTINUE  
         NNZ = NNZ + ICNT  
         IF (ICNT .NE. 1) GO TO 90
         IF (JX(INDX) .GT. 0) GO TO 90  
C
C        FOR DISTINCT COL. NO.S
C
         DO 80 II = 1, M  
            IF (JX(II) .EQ. J) GO TO 90  
   80    CONTINUE  
         JX(INDX) = J
   90    CONTINUE 
         MM = MM + ML  
  100 CONTINUE
C
C     FLAG THE SLACKS,FIRST CHECK FOR ALL EQUATIONS.
C
      IF (IALLEQ .EQ. 0) GO TO 200  
      DO 150 I = 1, M  
         IF (KT(I) .EQ. 0) GO TO 150
C
C        FOR DISTINCT COL. NO.S
C
         IF (JX(I) .GT. 0) GO TO 150  
         DO 130 II = 1, M 
            IF (JX(II) .EQ. (N+I)) GO TO 150  
  130    CONTINUE  
         JX(I) =  N + I
  150 CONTINUE  
C
  200 CONTINUE
      RETURN
C 
      END


C
C-------------------------------------------------------------------------
C


      SUBROUTINE LPDW00 (A,KT,RHS,COST,X,JX,PI)
C
C     OUTPUT REPORT ROUTINE FOR LPDUAL.  
C
C     LPDW10 OUTPUTS PROBLEM PARAMETERS.    
C     LPDW20 OUTPUTS PROBLEM DATA.
C     LPDW30 OUTPUTS HEADER FOR INTERMEDIATE OUTPUT.  
C     LPDW31 OUTPUTS LINE OF INTERMEDIATE OUTPUT.
C     LPDW32 OUTPUTS FINAL STATUS FOR INTERMEDIATE OUTPUT.
C     LPDW40 OUTPUTS SOLUTION REPORT.
C
C     LPDW00 IS A DUMMY ENTRY TO DEFINE ALL POSSIBLE PARAMETERS.
C
      INTEGER  KT,JX
      REAL  A,RHS,COST
      DOUBLE PRECISION  X,PI
      DIMENSION  A(*),KT(*),RHS(*),COST(*),X(*),JX(*),PI(*)
C
C
      INTEGER  ICON,INCOL,INPUTS,IO1,IO2,IO3,IOUTS,IPIV,IROW,IROW2,ITRY,
     1   IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,NCOST,NEQL,NUMNZ,
     2   ML,NL,M,N,MINMAX,LENWS,INFLAG,IALLEQ,KSLACK,NCUT,NEQ,NVER,
     3   ITRYO,IPRNT,IPDATA,ILINE,JFINAL,JFILE,JWIDTH,MX,IADD,IDROP,
     4   K,ITER,INVC,NUMVR
      COMMON /LPDZZI/ ICON,INCOL,INPUTS(22),IO1,IO2,IO3,IOUTS(4),IPIV,
     1   IROW,IROW2,ITRY,IVER,JCOL,JOUT,LENREQ,M1,MLE,N1,NCALL,
     2   NCOST,NEQL,NUMNZ
      EQUIVALENCE  (INPUTS(1),ML), (INPUTS(2),NL), (INPUTS(3),M),
     1   (INPUTS(4),N), (INPUTS(5),MINMAX), (INPUTS(6),LENWS),
     2   (INPUTS(7),INFLAG), (INPUTS(8),IALLEQ), (INPUTS(9),KSLACK),
     3   (INPUTS(10),NCUT), (INPUTS(11),NEQ), (INPUTS(12),NVER),
     4   (INPUTS(13),ITRYO), (INPUTS(14),IPRNT), (INPUTS(15),IPDATA),
     5   (INPUTS(16),ILINE), (INPUTS(17),JFINAL), (INPUTS(18),JFILE), 
     6   (INPUTS(19),JWIDTH), (INPUTS(20),MX), (INPUTS(21),IADD),
     7   (INPUTS(22),IDROP), (IOUTS(1),K), (IOUTS(2),ITER),
     8   (IOUTS(3),INVC), (IOUTS(4),NUMVR)
C
      REAL  ERRS,TOLS,ROWMAX,ROWSUM,COLMAX,COLSUM,TPIV,TCOST,TFEAS,
     1   TX,TART
      COMMON /LPDZZR/ ERRS(4),TOLS(5)
      EQUIVALENCE  (ERRS(1),ROWMAX), (ERRS(2),ROWSUM), 
     1   (ERRS(3),COLMAX), (ERRS(4),COLSUM), (TOLS(1),TPIV),
     2   (TOLS(2),TCOST), (TOLS(3),TFEAS), (TOLS(4),TX), (TOLS(5),TART)
C
      DOUBLE PRECISION  DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
      COMMON /LPDZZD/ DJ1,EP,OB,OBVAL,RAT,XMOST,YJ1
C
      CHARACTER*64  TITLES,PFILES
      COMMON /LPDZZC/ TITLES,PFILES
C
      INTEGER  I,IBUFF,INDENT,ITEMS,I1,I2,J,JMAX,L,LC,L1,L2,MARGIN
      REAL  RBUFF
      DOUBLE PRECISION  DBUFF,OBVAL1 
      CHARACTER*64  IAM
      CHARACTER*56  BLANK
      CHARACTER*52  TXTINP
      CHARACTER*40  HEDING
      CHARACTER*32  INDJXX,STATUS
      CHARACTER*28  TXTTOL
      CHARACTER*24  INDPI
      CHARACTER*20  COLVAL
      CHARACTER*3  OPTTYP
      CHARACTER*1  CONTYP
C
      DIMENSION  CONTYP(3),DBUFF(5),HEDING(4),IBUFF(8),OPTTYP(2),
     1   RBUFF(6),STATUS(7),TXTINP(29),TXTTOL(5) 
C                                                                      
      DATA IAM  /                                                    
     1'		SIMPLEX  METHOD					         '
     2   /
      DATA BLANK 
     1   /  '                                                        ' /
      DATA HEDING /  '  P R O B L E M    P A R A M E T E R S  ',
     1               '        P R O B L E M    D A T A        ',
     2               ' I N T E R M E D I A T E    O U T P U T ', 
     3               '     S O L U T I O N    R E P O R T     ' /
      DATA (TXTINP(I),I=1,13) /
     1   'ROW DIMENSION OF A                                  ',
     2   'COLUMN DIMENSION OF A                               ',
     3   'NUMBER OF CONSTRAINTS                               ',
     4   'NUMBER OF VARIABLES                                 ', 
     5   '0 IMPLIES MINIMIZE, 1 IMPLIES MAXIMIZE              ',  
     6   'SIZE OF WORKSPACE ARRAY                             ',
     7   '0 IMPLIES NEW PROBLEM                               ',
     8   '1 IMPLIES NEW PROBLEM WITH STARTING BASIS           ',
     9   '2 IMPLIES CHANGES IN COST AND RHS                   ',
     A   '3 IMPLIES CHANGES IN RHS ONLY                       ',
     B   '4 IMPLIES CHANGES IN COST ONLY                      ',
     C   '0 IMPLIES ALL EQUALITIES                            ',
     D   '1 IMPLIES SLACK COSTS ASSIGNED                      ' /
      DATA (TXTINP(I),I=14,29) /
     E   'ITERATION LIMIT                                     ',
     F   'EQUAL OBJECTIVE LIMIT                               ',
     G   'INVERSION FREQUENCY                                 ',
     H   '1 IMPLIES REINVERT AT OPTIMUM                       ',
     I   '1 IMPLIES OUTPUT PROBLEM PARAMETERS                 ',
     J   '1 IMPLIES OUTPUT PROBLEM DATA WITH A IN DENSE FORM  ',
     K   '2 IMPLIES OUTPUT PROBLEM DATA WITH A IN SPARSE FORM ',
     L   '1 IMPLIES OUTPUT INTERMEDIATE OUTPUT                ',
     M   '1 IMPLIES OUTPUT SOLUTION REPORT WITHOUT PI VECTOR  ', 
     N   '2 IMPLIES OUTPUT SOLUTION REPORT WITH PI VECTOR     ',
     O   '1 IMPLIES ALL OUTPUT BUT PROB PAR AND SOL REP FILED ',
     P   '2 IMPLIES ALL OUTPUT FILED                          ', 
     Q   'MAXIMUM OUTPUT COLUMN WIDTH                         ',
     R   'CONSTRAINT ADDITION LIMIT FOR BASE PROBLEM          ',
     S   'IF POSITIVE, NUMBER OF CONSTRAINTS ADDED            ',
     T   'IF POSITIVE, INDEX OF CONSTRAINT TO BE DROPPED      ' /
      DATA (TXTTOL(I),I=1,5) /
     1   'PIVOT TOLERANCE             ',
     2   'REDUCED COSTS TOLERANCE     ',
     3   'FEASIBILITY TOLERANCE       ',
     4   'ROUND-OFF ERROR TOLERANCE   ',
     5   'LARGE NUMBER FOR ARTIFICIALS' /
C                
      DATA COLVAL / ' COLUMN    VALUE    ' /
      DATA INDJXX / '    INDEX   JX          X       ' /
      DATA INDPI  / '   INDEX        PI      ' /
      DATA (STATUS(I),I=1,7) /
     1   'SOLUTION IS OPTIMAL             ',
     2   'NO FEASIBLE SOLUTION            ',
     3   'UNBOUNDED SOLUTION              ',
     4   'ITERATION LIMIT REACHED         ',
     5   'EQUAL OBJECTIVE LIMIT REACHED   ',
     6   'PROBLEM DATA ERROR              ',
     7   'ALGORITHM ERROR                 ' /
C
      DATA CONTYP /  'G', 'E' ,'L' /
      DATA OPTTYP /  'MIN', 'MAX' /
C
C.......................................................................
C
      ENTRY LPDW10
C
C     OUTPUT PROBLEM PARAMETERS.
C
C     OUTPUT HEADING.
C
      MARGIN = JWIDTH/2 - 35
      WRITE (IO2,100) BLANK(1:MARGIN),IAM,NCALL,BLANK(1:MARGIN),TITLES,         
     1   BLANK(1:MARGIN),HEDING(1) 
  100 FORMAT ( / 1H1 // A,A64,'CALL',I4 / A,'TITLE - ',A64 ///
     1   A,16X,A40 / )
C
C     OUTPUT INPUT ARRAY VALUES.
C
      WRITE (IO2,105) BLANK(1:MARGIN)
  105 FORMAT ( / A,'INPUT VALUES')
      DO 110 I = 1, 7
         WRITE (IO2,115) BLANK(1:MARGIN),I,INPUTS(I),TXTINP(I)
  110 CONTINUE
  115 FORMAT (A,'INPUT(',I2,') =',I6,3X,A52)
      DO 120 I = 8, 11
         WRITE (IO2,125) BLANK(1:MARGIN),TXTINP(I)
  120 CONTINUE
  125 FORMAT (A,20X,A52)
      DO 130 I = 8, 15
         WRITE (IO2,115) BLANK(1:MARGIN),I,INPUTS(I),TXTINP(I+4)
  130 CONTINUE 
      WRITE (IO2,125) BLANK(1:MARGIN),TXTINP(20)
      DO 140 I = 16, 17
         WRITE (IO2,115) BLANK(1:MARGIN),I,INPUTS(I),TXTINP(I+5)
  140 CONTINUE
      WRITE (IO2,125) BLANK(1:MARGIN),TXTINP(23)
      I = 18
      WRITE (IO2,115) BLANK(1:MARGIN),I,INPUTS(18),TXTINP(24)
      WRITE (IO2,125) BLANK(1:MARGIN),TXTINP(25)
      DO 145 I = 19, 22
         WRITE (IO2,115) BLANK(1:MARGIN),I,INPUTS(I),TXTINP(I+7)
  145 CONTINUE  
C
C     OUTPUT TOL ARRAY VALUES.
C
      WRITE (IO2,150) BLANK(1:MARGIN)
  150 FORMAT ( / A,'TOL VALUES')
      DO 160 I = 1, 5
         WRITE (IO2,165) BLANK(1:MARGIN),I,TOLS(I),TXTTOL(I)
  160 CONTINUE
  165 FORMAT (A,'TOL(',I1,') =',1PE14.5,3X,A)
C
      WRITE (IO2,195)      
  195 FORMAT ( / )
C
      RETURN 
C
C.......................................................................
C
      ENTRY LPDW20 (A,KT,RHS,COST)  
C
C     OUTPUT PROBLEM DATA.
C
C     OUTPUT HEADING.
C
      MARGIN = JWIDTH/2 - 35
      WRITE (IO3,100) BLANK(1:MARGIN),IAM,NCALL,BLANK(1:MARGIN),TITLES, 
     1   BLANK(1:MARGIN),HEDING(2)
C
C     OUTPUT OPTIMIZATION.
C
      WRITE (IO3,202) BLANK(1:MARGIN),OPTTYP(MINMAX+1)
  202 FORMAT ( / A,30X,A3,'IMIZATION')
C
C     OUTPUT VARIABLES HEADER.
C
      IF (KSLACK .EQ. 1) GO TO 206
      WRITE (IO3,204)
  204 FORMAT ( / 1X,'VARIABLES')
      GO TO 210 
  206 CONTINUE
      WRITE (IO3,208)
  208 FORMAT ( / 1X,'VARIABLES (INCLUDING SLACKS)')
  210 CONTINUE
      ITEMS = JWIDTH / 13 
      DO 212 J = 1, NCOST, ITEMS
         JMAX = MIN0(NCOST,J+ITEMS-1)
         WRITE (IO3,214) (I,I=J,JMAX)
  212 CONTINUE
  214 FORMAT (1X,10(I9,4X))
C
C     OUTPUT COSTS.
C
      WRITE (IO3,220)
  220 FORMAT ( / 1X,'COST')
      DO 222 J = 1, NCOST, ITEMS
         JMAX = MIN0(NCOST,J+ITEMS-1)
         WRITE (IO3,224) (COST(I),I=J,JMAX)
  222 CONTINUE
  224 FORMAT (1X,10G13.6)
C
C     OUTPUT CONSTRAINT TYPES.
      WRITE (IO3,230)
  230 FORMAT ( / 1X,'CONSTRAINTS')
      DO 234 J = 1, M, ITEMS
         JMAX = MIN0(M,J+ITEMS-1)
         IF (IALLEQ .EQ. 0) GO TO 232
         WRITE (IO3,236) (I,CONTYP(KT(I)+2),I=J,JMAX)
         GO TO 234
  232    CONTINUE
         WRITE (IO3,236) (I,CONTYP(2),I=J,JMAX)
  234 CONTINUE
  236 FORMAT (1X,10(I5,'  ( ',A1,' ) '))
C
C     OUTPUT RIGHT-HAND-SIDE.
C
      WRITE (IO3,240)
  240 FORMAT ( / 1X,'RHS')
      DO 242 J = 1, M, ITEMS
         JMAX = MIN0(M,J+ITEMS-1)
         WRITE (IO3,224) (RHS(I),I=J,JMAX)
  242 CONTINUE
C 
C     OUTPUT A-MATRIX.
C
      WRITE (IO3,250)
  250 FORMAT ( / 1X,'A-MATRIX')
      IF (IPDATA .EQ. 2) GO TO 270
C
C     DENSE FORM 
C
      WRITE (IO3,260) BLANK(1:MARGIN)
  260 FORMAT (A,32X,'COLUMNS')
      DO 262 J = 1, N, ITEMS
         JMAX = MIN0(N,J+ITEMS-1)
         WRITE (IO3,214) (I,I=J,JMAX)
  262 CONTINUE
  264 FORMAT (1X,'ROW',I4)
      DO 268 L1 = 1, M 
         WRITE (IO3,264) L1
         L2 = ML*(N-1) + L1
         I1 = L1
         DO 266 J = 1, N, ITEMS
            I2 = MIN0(L2,I1+ML*(ITEMS-1))
            WRITE (IO3,224) (A(L),L=I1,I2,ML)
            I1 = I2 + ML
  266    CONTINUE    
  268 CONTINUE
      GO TO 290
C
C     SPARSE FORM
C
  270 CONTINUE 
      WRITE (IO3,272) BLANK(1:MARGIN)
  272 FORMAT (A,27X,'NON-ZERO ELEMENTS')
      ITEMS = (JWIDTH - 7) / 20
      WRITE (IO3,274) (COLVAL,I=1,ITEMS)
  274 FORMAT ( / 8X,6A20)
      DO 282 L1 = 1, M 
         WRITE (IO3,264) L1
         L = L1
         I1 = 0 
         DO 280 J = 1, N 
            IF (ABS(A(L)) .LT. TX) GO TO 278
            I1 = I1 + 1
            IBUFF(I1) = J
            RBUFF(I1) = A(L)
            IF (I1 .LT. ITEMS) GO TO 278
            WRITE (IO3,276) (IBUFF(I),RBUFF(I),I=1,ITEMS)
  276       FORMAT (8X,6(I7,G13.6))
            I1 = 0
  278       CONTINUE
            L = L + ML
  280    CONTINUE  
         IF (I1 .GT. 0) WRITE (IO3,276) (IBUFF(I),RBUFF(I),I=1,I1)     
  282 CONTINUE    
C
  290 CONTINUE
      WRITE (IO3,195)
C
      RETURN
C
C.......................................................................
C
      ENTRY LPDW30  
C
C     OUTPUT HEADER FOR INTERMEDIATE OUTPUT.
C
C     OUTPUT HEADING.
C
      MARGIN = JWIDTH/2 - 35
      WRITE (IO3,100) BLANK(1:MARGIN),IAM,NCALL,BLANK(1:MARGIN),TITLES, 
     1   BLANK(1:MARGIN),HEDING(3) 
C
C     SET CONTINUATION FLAG.
C
      ICON = 1
C
      RETURN
C
C.......................................................................
C
      ENTRY LPDW31 
C
C     OUTPUT LINE OF INTERMEDIATE OUTPUT.
C
C     GET OBJECTIVE OF ORIGINAL PROBLEM.
C
      IF (MINMAX .EQ. 1) GO TO 340
      OBVAL1 = OBVAL
      GO TO 345
  340 CONTINUE 
      OBVAL1 = -OBVAL
  345 CONTINUE
C
C     CHECK FOR RECENT INVERSION.
C
      IF (INVC .LE. 1 .AND. NUMVR .GT. 0) GO TO 350
      GO TO 355
C
C     OUTPUT INVERSION NO.
C
  350 CONTINUE
      WRITE (IO3,352) BLANK(1:MARGIN),NUMVR
  352 FORMAT ( / A,1X,'INVERSION NO.',I5)
  355 CONTINUE
      IF (K .GE. 1) GO TO 370
      IF (INVC .EQ. 1 .OR. ICON .EQ. 1) GO TO 360
      GO TO 365
C
C     OUTPUT ITERATION HEADER.
C
  360 CONTINUE
      WRITE (IO3,362) BLANK(1:MARGIN),BLANK(1:MARGIN)
  362 FORMAT ( / A,'  ITER-',6X,'OBJECTIVE',5X,'MOST NEG-',4X,'IN',
     1   7X,'IN',5X,'OUT',5X,'PIVOT' /
     2   A,'  ATION',8X,'VALUE',7X,'ATIVE  X',5X,'ROW',
     3   3X,'COLUMN',2X,'COLUMN',4X,'RATIO')
      LC = 20 
      ICON = 0
C
C     OUTPUT LINE OF ITERATION INFO.
C
  365 CONTINUE
      WRITE (IO3,367) BLANK(1:MARGIN),ITER,OBVAL1,XMOST,
     1   IROW,INCOL,JOUT,RAT
  367 FORMAT (A,I7,1PE17.7,1PE12.2,0PI7,0P2I8,1PE12.2)
      LC = LC - 1
      IF (LC .EQ. 0) ICON = 1
C
  370 CONTINUE
C
      RETURN
C
C.......................................................................
C
      ENTRY LPDW32
C
C     OUTPUT FINAL STATUS FOR INTERMEDIATE OUTPUT.
C
      IF (K .GT. 1) GO TO 385
      WRITE (IO3,380) BLANK(1:MARGIN),OPTTYP(MINMAX+1),OBVAL 
  380 FORMAT ( / A,1X,'SOLUTION IS OPTIMAL, ',A3,
     1   '. OBJECTIVE VALUE =',1PE15.7)
      GO TO 390
  385 CONTINUE
      WRITE (IO3,387) BLANK(1:MARGIN),STATUS(K)
  387 FORMAT ( / A,1X,A32)
C
  390 CONTINUE
C
      RETURN
C
C.......................................................................
C
      ENTRY LPDW40 (X,JX,PI)
C
C     OUTPUT SOLUTION REPORT.
C
C     OUTPUT HEADING.
C
      MARGIN = JWIDTH/2 - 35
      WRITE (IO2,100) BLANK(1:MARGIN),IAM,NCALL,BLANK(1:MARGIN),TITLES, 
     1   BLANK(1:MARGIN),HEDING(4)
C
C     OUTPUT STATUS.
C
      WRITE (IO2,400) BLANK(1:MARGIN),STATUS(K)
  400 FORMAT ( / A,18X,A32)
C
C     OUTPUT OPTIMAL OBJECTIVE VALUE. 
C
      IF (DABS(OBVAL) .GE. 1.0D+6 .OR. DABS(OBVAL) .LT. 1.0D-3)
     1   GO TO 410
      WRITE (IO2,405) BLANK(1:MARGIN),OPTTYP(MINMAX+1),OBVAL
  405 FORMAT (A,18X,A3,'IMUM OBJECTIVE VALUE',3X,F16.6)
      GO TO 420
  410 CONTINUE
      WRITE (IO2,415) BLANK(1:MARGIN),OPTTYP(MINMAX+1),OBVAL
  415 FORMAT (A,18X,A3,'IMUM OBJECTIVE VALUE',3X,1PE16.9)
C
C     OUTPUT LINES OF STATISTICS.
C
  420 CONTINUE
      WRITE (IO2,425) BLANK(1:MARGIN),ITER,
     1 		      BLANK(1:MARGIN),NUMVR
  425 FORMAT (A,18X,'NUMBER OF ITERATIONS  ',0PI9/  
     1        A,18X,'NUMBER OF INVERSIONS  ',0PI9)
C
C     OUTPUT SOLUTION VECTOR.
C
      IF (K .EQ. 2 .OR. K .EQ. 3) GO TO 490
      ITEMS = MIN0(M,JWIDTH/32)
      INDENT = (JWIDTH-32*ITEMS)/2 + 1
      WRITE (IO2,430) BLANK(1:MARGIN)
  430 FORMAT ( /// A,28X,'SOLUTION VECTOR')
      WRITE (IO2,432) BLANK(1:INDENT), (INDJXX,I=1,ITEMS)
  432 FORMAT ( / A,4A32)
      L = 2
      DO 434 J = 1, M
         IF (DABS(X(J)) .GE. 1.0D+6) GO TO 436
         IF (DABS(X(J)) .GE. 1.0D-3) L = 1
  434 CONTINUE
      GO TO 440
  436 CONTINUE
      L = 2
  440 CONTINUE
      I2 = 20
      DO 458 J = 1, M, ITEMS
         JMAX = MIN0(M,J+ITEMS-1)
         I1 = 0
         DO 445 L1 = J, JMAX
            I1 = I1 + 1
            IBUFF(I1) = L1
            IBUFF(I1+4) = JX(L1)
            DBUFF(I1) = X(L1)
  445    CONTINUE
         I2 = I2 - 1
         IF (I2 .GT. 0) GO TO 448
         IF (JMAX .LT. M) WRITE (IO2,432) BLANK(1:INDENT),
     1      (INDJXX,I=1,ITEMS)
         I2 = 20
  448    CONTINUE
         GO TO (450, 454), L
  450    CONTINUE
         WRITE (IO2,452) BLANK(1:INDENT), (IBUFF(I),IBUFF(I+4),
     1      DBUFF(I),I=1,I1)
  452    FORMAT (A,4(I8,I6,F15.6,3X))
         GO TO 458
  454    CONTINUE
         WRITE (IO2,456) BLANK(1:INDENT), (IBUFF(I),IBUFF(I+4),
     1      DBUFF(I),I=1,I1)
  456    FORMAT (A,4(I8,I6,1PE15.7,3X))
  458 CONTINUE
C
C     OUTPUT SHADOW PRICES IF REQUESTED. 
C 
      IF (JFINAL .NE. 2) GO TO 490
      ITEMS = MIN0(M,JWIDTH/24)
      INDENT = (JWIDTH-24*ITEMS)/2 + 1
      WRITE (IO2,460) BLANK(1:MARGIN)
  460 FORMAT ( /// A,29X,'SHADOW PRICES')
      WRITE (IO2,462) BLANK(1:INDENT), (INDPI,I=1,ITEMS)
  462 FORMAT ( / A,5A24)
      L = 2
      DO 464 J = 1, M
         IF (DABS(PI(J)) .GE. 1.0D+6) GO TO 466
         IF (DABS(PI(J)) .GE. 1.0D-3) L = 1
  464 CONTINUE
      GO TO 470
  466 CONTINUE
      L = 2
  470 CONTINUE
      I2 = 20
      DO 488 J = 1, M, ITEMS
         JMAX = MIN0(M,J+ITEMS-1)
         I1 = 0
         DO 475 L1 = J, JMAX
            I1 = I1 + 1
            IBUFF(I1) = L1
            DBUFF(I1) = PI(L1)
  475    CONTINUE
         I2 = I2 - 1
         IF (I2 .GT. 0) GO TO 478
         IF (JMAX .LT. M) WRITE (IO2,462) BLANK(1:INDENT),
     1      (INDPI,I=1,ITEMS)
         I2 = 20
  478    CONTINUE
         GO TO (480, 484), L
  480    CONTINUE
         WRITE (IO2,482) BLANK(1:INDENT), (IBUFF(I),DBUFF(I),I=1,I1)
  482    FORMAT (A,5(I8,F15.6,1X))
         GO TO 488
  484    CONTINUE
         WRITE (IO2,486) BLANK(1:INDENT), (IBUFF(I),DBUFF(I),I=1,I1)
  486    FORMAT (A,5(I8,1PE15.7,1X))
  488 CONTINUE
C
  490 CONTINUE
      WRITE (IO2,495) 
  495 FORMAT (//) 
C
      RETURN
C
      END
